@@ -39,8 +39,16 @@
 #include "kjs_dom.lut.h"
 #include "khtmlpart_p.h"
 
+#include "html_objectimpl.h"
+
+#if APPLE_CHANGES
+#include <JavaScriptCore/runtime_object.h>
+#endif
+
 using namespace KJS;
 
+using DOM::DOMException;
+
 // -------------------------------------------------------------------------
 /* Source for DOMNodeProtoTable. Use "make hashtables" to regenerate.
 @begin DOMNodeProtoTable 13
@@ -59,6 +67,8 @@ using namespace KJS;
   removeEventListener	DOMNode::RemoveEventListener	DontDelete|Function 3
   dispatchEvent		DOMNode::DispatchEvent	DontDelete|Function 1
   contains	DOMNode::Contains		DontDelete|Function 1
+# "DOM level 0" (from Gecko DOM reference; also in WinIE)
+  item          DOMNode::Item           DontDelete|Function 1
 @end
 */
 DEFINE_PROTOTYPE("DOMNode",DOMNodeProto)
@@ -105,6 +115,7 @@ bool DOMNode::toBoolean(ExecState *) const
   onblur	DOMNode::OnBlur			DontDelete
   onchange	DOMNode::OnChange		DontDelete
   onclick	DOMNode::OnClick		DontDelete
+  oncontextmenu	DOMNode::OnContextMenu		DontDelete
   ondblclick	DOMNode::OnDblClick		DontDelete
   ondragdrop	DOMNode::OnDragDrop		DontDelete
   onerror	DOMNode::OnError		DontDelete
@@ -121,6 +132,7 @@ bool DOMNode::toBoolean(ExecState *) const
   onmove	DOMNode::OnMove			DontDelete
   onreset	DOMNode::OnReset		DontDelete
   onresize	DOMNode::OnResize		DontDelete
+  onscroll      DOMNode::OnScroll               DontDelete
   onselect	DOMNode::OnSelect		DontDelete
   onsubmit	DOMNode::OnSubmit		DontDelete
   onunload	DOMNode::OnUnload		DontDelete
@@ -187,6 +199,8 @@ Value DOMNode::getValueProperty(ExecState *exec, int token) const
     return getListener(DOM::EventImpl::CHANGE_EVENT);
   case OnClick:
     return getListener(DOM::EventImpl::KHTML_CLICK_EVENT);
+  case OnContextMenu:
+    return getListener(DOM::EventImpl::CONTEXTMENU_EVENT);
   case OnDblClick:
     return getListener(DOM::EventImpl::KHTML_DBLCLICK_EVENT);
   case OnDragDrop:
@@ -196,11 +210,11 @@ Value DOMNode::getValueProperty(ExecState *exec, int token) const
   case OnFocus:
     return getListener(DOM::EventImpl::FOCUS_EVENT);
   case OnKeyDown:
-    return getListener(DOM::EventImpl::KHTML_KEYDOWN_EVENT);
+    return getListener(DOM::EventImpl::KEYDOWN_EVENT);
   case OnKeyPress:
-    return getListener(DOM::EventImpl::KHTML_KEYPRESS_EVENT);
+    return getListener(DOM::EventImpl::KEYPRESS_EVENT);
   case OnKeyUp:
-    return getListener(DOM::EventImpl::KHTML_KEYUP_EVENT);
+    return getListener(DOM::EventImpl::KEYUP_EVENT);
   case OnLoad:
     return getListener(DOM::EventImpl::LOAD_EVENT);
   case OnMouseDown:
@@ -219,6 +233,8 @@ Value DOMNode::getValueProperty(ExecState *exec, int token) const
     return getListener(DOM::EventImpl::RESET_EVENT);
   case OnResize:
     return getListener(DOM::EventImpl::RESIZE_EVENT);
+  case OnScroll:
+    return getListener(DOM::EventImpl::SCROLL_EVENT);
   case OnSelect:
     return getListener(DOM::EventImpl::SELECT_EVENT);
   case OnSubmit:
@@ -300,6 +316,9 @@ void DOMNode::putValue(ExecState *exec, int token, const Value& value, int /*att
   case OnClick:
     setListener(exec,DOM::EventImpl::KHTML_CLICK_EVENT,value);
     break;
+  case OnContextMenu:
+    setListener(exec,DOM::EventImpl::CONTEXTMENU_EVENT,value);
+    break;
   case OnDblClick:
     setListener(exec,DOM::EventImpl::KHTML_DBLCLICK_EVENT,value);
     break;
@@ -313,13 +332,13 @@ void DOMNode::putValue(ExecState *exec, int token, const Value& value, int /*att
     setListener(exec,DOM::EventImpl::FOCUS_EVENT,value);
     break;
   case OnKeyDown:
-    setListener(exec,DOM::EventImpl::KHTML_KEYDOWN_EVENT,value);
+    setListener(exec,DOM::EventImpl::KEYDOWN_EVENT,value);
     break;
   case OnKeyPress:
-    setListener(exec,DOM::EventImpl::KHTML_KEYPRESS_EVENT,value);
+    setListener(exec,DOM::EventImpl::KEYPRESS_EVENT,value);
     break;
   case OnKeyUp:
-    setListener(exec,DOM::EventImpl::KHTML_KEYUP_EVENT,value);
+    setListener(exec,DOM::EventImpl::KEYUP_EVENT,value);
     break;
   case OnLoad:
     setListener(exec,DOM::EventImpl::LOAD_EVENT,value);
@@ -348,6 +367,8 @@ void DOMNode::putValue(ExecState *exec, int token, const Value& value, int /*att
   case OnResize:
     setListener(exec,DOM::EventImpl::RESIZE_EVENT,value);
     break;
+  case OnScroll:
+    setListener(exec,DOM::EventImpl::SCROLL_EVENT,value);
   case OnSelect:
     setListener(exec,DOM::EventImpl::SELECT_EVENT,value);
     break;
@@ -359,13 +380,13 @@ void DOMNode::putValue(ExecState *exec, int token, const Value& value, int /*att
     break;
   case ScrollTop: {
     khtml::RenderObject *rend = node.handle() ? node.handle()->renderer() : 0L;
-    if (rend && rend->layer())
+    if (rend && rend->layer() && rend->style()->hidesOverflow())
         rend->layer()->scrollToYOffset(value.toInt32(exec));
     break;
   }
   case ScrollLeft: {
     khtml::RenderObject *rend = node.handle() ? node.handle()->renderer() : 0L;
-    if (rend && rend->layer())
+    if (rend && rend->layer() && rend->style()->hidesOverflow())
       rend->layer()->scrollToXOffset(value.toInt32(exec));
     break;
   }
@@ -405,8 +426,9 @@ void DOMNode::setListener(ExecState *exec, int eventId, Value func) const
 Value DOMNode::getListener(int eventId) const
 {
     DOM::EventListener *listener = node.handle()->getHTMLEventListener(eventId);
-    if (listener)
-	return static_cast<JSEventListener*>(listener)->listenerObj();
+    JSEventListener *jsListener = static_cast<JSEventListener*>(listener);
+    if ( jsListener && jsListener->listenerObjImp() )
+	return jsListener->listenerObj();
     else
 	return Null();
 }
@@ -467,7 +489,10 @@ Value DOMNodeProtoFunc::tryCall(ExecState *exec, Object &thisObj, const List &ar
 	    bool retval = !impl->checkNoOwner(other.handle(),exceptioncode);
 	    return Boolean(retval && exceptioncode == 0);
 	}
+        return Undefined();
     }
+    case DOMNode::Item:
+      return getDOMNode(exec, node.childNodes().item(static_cast<unsigned long>(args[0].toNumber(exec))));
   }
 
   return Undefined();
@@ -482,6 +507,16 @@ DOMNodeList::~DOMNodeList()
   ScriptInterpreter::forgetDOMObject(list.handle());
 }
 
+Value DOMNodeList::toPrimitive(ExecState *exec, Type /*preferred*/) const
+{
+  if (list.isNull())
+    return Null();
+
+  return String(toString(exec));
+}
+
+
+
 // We have to implement hasProperty since we don't use a hashtable for 'length' and 'item'
 // ## this breaks "for (..in..)" though.
 bool DOMNodeList::hasProperty(ExecState *exec, const Identifier &p) const
@@ -725,9 +760,9 @@ Value DOMDocument::getValueProperty(ExecState *exec, int token) const
   case ReadyState:
     {
     DOM::DocumentImpl* docimpl = node.handle()->getDocument();
-    if ( docimpl && docimpl->view() )
+    if ( docimpl )
     {
-      KHTMLPart* part = docimpl->view()->part();
+      KHTMLPart* part = docimpl->part();
       if ( part ) {
         if (part->d->m_bComplete) return String("complete");
         if (docimpl->parsing()) return String("loading");
@@ -1287,8 +1322,8 @@ bool KJS::checkNodeSecurity(ExecState *exec, const DOM::Node& n)
     return false;
 
   // Check to see if the currently executing interpreter is allowed to access the specified node
-  KHTMLView *view = n.handle()->getDocument()->view();
-  Window* win = view && view->part() ? Window::retrieveWindow(view->part()) : 0L;
+  KHTMLPart *part = n.handle()->getDocument()->part();
+  Window* win = part ? Window::retrieveWindow(part) : 0L;
   if ( !win || !win->isSafeScript(exec) )
     return false;
   return true;
@@ -1358,6 +1393,19 @@ Value KJS::getDOMNamedNodeMap(ExecState *exec, const DOM::NamedNodeMap &m)
   return Value(cacheDOMObject<DOM::NamedNodeMap, KJS::DOMNamedNodeMap>(exec, m));
 }
 
+Value KJS::getRuntimeObject(ExecState *exec, const DOM::Node &node)
+{
+    DOM::HTMLElement element = static_cast<DOM::HTMLElement>(node);
+    DOM::HTMLAppletElementImpl *appletElement = static_cast<DOM::HTMLAppletElementImpl *>(element.handle());
+    
+    if (appletElement->getAppletInstance()) {
+        // The instance is owned by the applet element.
+        RuntimeObjectImp *appletImp = new RuntimeObjectImp(appletElement->getAppletInstance(), false);
+        return Value(appletImp);
+    }
+    return Undefined();
+}
+
 Value KJS::getDOMNodeList(ExecState *exec, const DOM::NodeList &l)
 {
   return Value(cacheDOMObject<DOM::NodeList, KJS::DOMNodeList>(exec, l));
@@ -1624,24 +1672,32 @@ Value DOMCharacterDataProtoFunc::tryCall(ExecState *exec, Object &thisObj, const
   }
   DOM::CharacterData data = static_cast<DOMCharacterData *>(thisObj.imp())->toData();
   switch(id) {
-    case DOMCharacterData::SubstringData:
-      return getStringOrNull(data.substringData(args[0].toInteger(exec),args[1].toInteger(exec)));
+    case DOMCharacterData::SubstringData: {
+      const int count = args[1].toInteger(exec);
+      if (count < 0)
+        throw DOMException(DOMException::INDEX_SIZE_ERR);
+      return getStringOrNull(data.substringData(args[0].toInteger(exec), count));
+    }
     case DOMCharacterData::AppendData:
       data.appendData(args[0].toString(exec).string());
       return Undefined();
-      break;
     case DOMCharacterData::InsertData:
-      data.insertData(args[0].toInteger(exec),args[1].toString(exec).string());
-      return  Undefined();
-      break;
-    case DOMCharacterData::DeleteData:
-      data.deleteData(args[0].toInteger(exec),args[1].toInteger(exec));
-      return  Undefined();
-      break;
-    case DOMCharacterData::ReplaceData:
-      data.replaceData(args[0].toInteger(exec),args[1].toInteger(exec),args[2].toString(exec).string());
+      data.insertData(args[0].toInteger(exec), args[1].toString(exec).string());
       return Undefined();
-      break;
+    case DOMCharacterData::DeleteData: {
+      const int count = args[1].toInteger(exec);
+      if (count < 0)
+        throw DOMException(DOMException::INDEX_SIZE_ERR);
+      data.deleteData(args[0].toInteger(exec), count);
+      return Undefined();
+    }
+    case DOMCharacterData::ReplaceData: {
+      const int count = args[1].toInteger(exec);
+      if (count < 0)
+        throw DOMException(DOMException::INDEX_SIZE_ERR);
+      data.replaceData(args[0].toInteger(exec), count, args[2].toString(exec).string());
+      return Undefined();
+    }
     default:
       return Undefined();
   }
