@@ -107,7 +107,7 @@ const unsigned short DOM::tagPriority[] = {
     3, // ID_MARQUEE
     5, // ID_MENU
     0, // ID_META
-    1, // ID_NOBR
+    4, // ID_NOBR
    10,// ID_NOEMBED
    10,// ID_NOFRAMES
     3, // ID_NOSCRIPT
@@ -588,14 +588,15 @@ bool DOM::checkChild(ushort tagID, ushort childID)
     case ID_WBR:
         return check_array(childID, tag_list_1);
     case ID_P:
+        // P: ( _0 | TABLE ) *
+        return check_array(childID, tag_list_0) || childID == ID_TABLE;
     case ID_H1:
     case ID_H2:
     case ID_H3:
     case ID_H4:
     case ID_H5:
     case ID_H6:
-        if (childID == ID_TABLE || (tagID != ID_P && childID == ID_CENTER)) return true;
-        return check_array(childID, tag_list_0);
+        return check_array(childID, tag_list_1) && (childID < ID_H1 || childID > ID_H6);
     case ID_BASEFONT:
     case ID_BR:
     case ID_AREA:
@@ -614,12 +615,10 @@ bool DOM::checkChild(ushort tagID, ushort childID)
         return false;
     case ID_BODY:
         // BODY: _1 * + _2
-        if( check_array(childID, tag_list_1) ) return true;
-        return check_array(childID, tag_list_2);
+        return check_array(childID, tag_list_1) || check_array(childID, tag_list_2);
     case ID_ADDRESS:
         // ADDRESS: ( _0 | P ) *
-        if( check_array(childID, tag_list_0) ) return true;
-        return (childID == ID_P);
+        return check_array(childID, tag_list_0) || childID == ID_P;
     case ID_LI:
     case ID_DT:
     case ID_DIV:
@@ -639,6 +638,8 @@ bool DOM::checkChild(ushort tagID, ushort childID)
     case ID_MARQUEE:
     case ID_UL:
     case ID_OL:
+    case ID_DIR:
+    case ID_MENU:
         // DIV: _1 *
         return check_array(childID, tag_list_1);
     case ID_MAP:
@@ -660,11 +661,6 @@ bool DOM::checkChild(ushort tagID, ushort childID)
     case ID_DL:
         // DL: _6 +
         return check_array(childID, tag_list_1);
-    case ID_DIR:
-    case ID_MENU:
-        // (DIR|MENU): LI + - _3
-        if(childID == ID_LI) return true;
-        return false;
     case ID_FORM:
         // FORM: _1 * - FORM
         return check_array(childID, tag_list_1);
