@@ -228,7 +228,7 @@ int InlineFlowBox::placeBoxesHorizontally(int x)
     
     for (InlineBox* curr = firstChild(); curr; curr = curr->nextOnLine()) {
         if (curr->object()->isText()) {
-            TextRun* text = static_cast<TextRun*>(curr);
+            InlineTextBox* text = static_cast<InlineTextBox*>(curr);
             text->setXPos(x);
             x += text->width();
         }
@@ -279,7 +279,8 @@ void InlineFlowBox::verticallyAlignBoxes(int& heightOfBlock)
     int maxAscent = 0;
     int maxDescent = 0;
 
-    // Figure out if we're in strict mode.
+    // Figure out if we're in strict mode.  Note that we can't simply use !style()->htmlHacks(),
+    // because that would match almost strict mode as well.
     RenderObject* curr = object();
     while (curr && !curr->element())
         curr = curr->container();
@@ -336,14 +337,14 @@ void InlineFlowBox::computeLogicalBoxHeights(int& maxPositionTop, int& maxPositi
 {
     if (isRootInlineBox()) {
         // Examine our root box.
-        setHeight(object()->lineHeight(m_firstLine));
+        setHeight(object()->lineHeight(m_firstLine, true));
         bool isTableCell = object()->isTableCell();
         if (isTableCell) {
             RenderTableCell* tableCell = static_cast<RenderTableCell*>(object());
-            setBaseline(tableCell->RenderBlock::baselinePosition(m_firstLine));
+            setBaseline(tableCell->RenderBlock::baselinePosition(m_firstLine, true));
         }
         else
-            setBaseline(object()->baselinePosition(m_firstLine));
+            setBaseline(object()->baselinePosition(m_firstLine, true));
         if (hasTextChildren() || strictMode) {
             int ascent = baseline();
             int descent = height() - ascent;
@@ -413,7 +414,7 @@ void InlineFlowBox::placeBoxesVertically(int y, int maxHeight, int maxAscent, bo
         int newY = curr->yPos();
         int newHeight = curr->height();
         int newBaseline = curr->baseline();
-        if (curr->isTextRun() || curr->isInlineFlowBox()) {
+        if (curr->isInlineTextBox() || curr->isInlineFlowBox()) {
             const QFontMetrics &fm = curr->object()->fontMetrics( m_firstLine );
             newBaseline = fm.ascent();
             newY += curr->baseline() - newBaseline;
@@ -498,7 +499,8 @@ void InlineFlowBox::paintBackgroundAndBorder(QPainter *p, int _x, int _y,
     // You can use p::first-line to specify a background. If so, the root line boxes for
     // a line may actually have to paint a background.
     RenderStyle* styleToUse = object()->style(m_firstLine);
-    if (object()->hasFirstLine() || (parent() && object()->shouldPaintBackgroundOrBorder())) {
+    if ((!parent() && m_firstLine && styleToUse != object()->style()) || 
+        (parent() && object()->shouldPaintBackgroundOrBorder())) {
         CachedImage* bg = styleToUse->backgroundImage();
         bool hasBackgroundImage = bg && (bg->pixmap_size() == bg->valid_rect().size()) &&
                                   !bg->isTransparent() && !bg->isErrorImage();
@@ -563,6 +565,17 @@ void InlineFlowBox::paintDecorations(QPainter *p, int _x, int _y,
     RenderStyle* styleToUse = object()->style(m_firstLine);
     int deco = parent() ? styleToUse->textDecoration() : styleToUse->textDecorationsInEffect();
     if (deco != TDNONE && shouldDrawDecoration(object())) {
+#if APPLE_CHANGES
+        // Set up the appropriate text-shadow effect for the decoration.
+        // FIXME: Support multiple shadow effects.  Need more from the CG API before we can do this.
+        bool setShadow = false;
+        if (styleToUse->textShadow()) {
+            p->setShadow(styleToUse->textShadow()->x, styleToUse->textShadow()->y,
+                         styleToUse->textShadow()->blur, styleToUse->textShadow()->color);
+            setShadow = true;
+        }
+#endif
+        
         // We must have child boxes and have decorations defined.
         _tx += borderLeft() + paddingLeft();
         int w = m_width - (borderLeft() + paddingLeft() + borderRight() + paddingRight());
@@ -582,5 +595,10 @@ void InlineFlowBox::paintDecorations(QPainter *p, int _x, int _y,
             p->setPen(linethrough);
             p->drawLineForText(_tx, _ty, 2*m_baseline/3, w);
         }
+
+#if APPLE_CHANGES
+        if (setShadow)
+            p->clearShadow();
+#endif
     }
 }
