@@ -221,6 +221,8 @@ static void freeHandle(void *free);
 
 static const int caseDelta = ('a' - 'A');
 
+const char * const QString::null = 0;
+
 KWQStringData *QString::shared_null = 0;
 KWQStringData **QString::shared_null_handle = 0;
 
@@ -597,6 +599,29 @@ QChar *KWQStringData::makeUnicode()
 // -------------------------------------------------------------------------
 
 
+static inline bool compareIgnoringCaseForASCIIOnly(char c1, char c2)
+{
+    if (c2 >= 'a' && c2 <= 'z') {
+        return c1 == c2 || c1 == c2 - caseDelta;
+    }
+    if (c2 >= 'A' && c2 <= 'Z') {
+        return c1 == c2 || c1 == c2 + caseDelta;
+    }
+    return c1 == c2;
+}
+
+static inline bool compareIgnoringCaseForASCIIOnly(QChar c1, char c2)
+{
+    if (c2 >= 'a' && c2 <= 'z') {
+        return c1 == c2 || c1.unicode() == c2 - caseDelta;
+    }
+    if (c2 >= 'A' && c2 <= 'Z') {
+        return c1 == c2 || c1.unicode() == c2 + caseDelta;
+    }
+    return c1 == c2;
+}
+
+
 QString QString::number(int n)
 {
     QString qs;
@@ -678,6 +703,9 @@ QString QString::fromNSString(NSString *nss)
 
 NSString *QString::getNSString() const
 {
+    // The Cocoa calls in this method don't need exceptions blocked
+    // because they are simple NSString calls that can't throw.
+
     if (dataHandle[0]->_isUnicodeValid) {
         return [NSString stringWithCharacters:(const unichar *)unicode() length:dataHandle[0]->_length];
     }
@@ -942,6 +970,53 @@ bool QString::startsWith( const QString& s ) const
     return TRUE;
 }
 
+bool QString::startsWith(const char *prefix) const
+{
+    int prefixLength = strlen(prefix);
+    if (dataHandle[0]->_isAsciiValid) {
+        return strncmp(prefix, ascii(), prefixLength) == 0;
+    } else if (dataHandle[0]->_isUnicodeValid) {
+        int l = dataHandle[0]->_length;
+        if (prefixLength > l) {
+            return false;
+        }
+        const QChar *uni = unicode();        
+        for (int i = 0; i < prefixLength; ++i) {
+            if (uni[i] != prefix[i]) {
+                return false;
+            }
+        }
+    } else {
+        FATAL("invalid character cache");
+    }
+    return true;
+}
+
+bool QString::startsWith(const char *prefix, bool caseSensitive) const
+{
+    if (caseSensitive) {
+        return startsWith(prefix);
+    }
+    int prefixLength = strlen(prefix);
+    if (dataHandle[0]->_isAsciiValid) {
+        return strncasecmp(prefix, ascii(), prefixLength) == 0;
+    } else if (dataHandle[0]->_isUnicodeValid) {
+        int l = dataHandle[0]->_length;
+        if (prefixLength > l) {
+            return false;
+        }
+        const QChar *uni = unicode();        
+        for (int i = 0; i < prefixLength; ++i) {
+            if (!compareIgnoringCaseForASCIIOnly(uni[i], prefix[i])) {
+                return false;
+            }
+        }
+    } else {
+        FATAL("invalid character cache");
+    }
+    return true;
+}
+
 bool QString::endsWith( const QString& s ) const
 {
     const QChar *uni = unicode();
@@ -1078,29 +1153,6 @@ int QString::find(const QString &str, int index, bool caseSensitive) const
 }
 
 
-static inline bool compareIgnoringCaseForASCIIOnly(char c1, char c2)
-{
-    if (c2 >= 'a' && c2 <= 'z') {
-        return c1 == c2 || c1 == c2 - caseDelta;
-    }
-    if (c2 >= 'A' && c2 <= 'Z') {
-        return c1 == c2 || c1 == c2 + caseDelta;
-    }
-    return c1 == c2;
-}
-
-static inline bool compareIgnoringCaseForASCIIOnly(QChar c1, char c2)
-{
-    if (c2 >= 'a' && c2 <= 'z') {
-        return c1 == c2 || c1.unicode() == c2 - caseDelta;
-    }
-    if (c2 >= 'A' && c2 <= 'Z') {
-        return c1 == c2 || c1.unicode() == c2 + caseDelta;
-    }
-    return c1 == c2;
-}
-
-
 // This function should be as fast as possible, every little bit helps.
 // Our usage patterns are typically small strings.  In time trials
 // this simplistic algorithm is much faster than Boyer-Moore or hash
@@ -2304,7 +2356,7 @@ QString &QString::replace(const QRegExp &qre, const QString &str)
     int slen  = str.dataHandle[0]->_length;
     int len;
     while ( index < (int)dataHandle[0]->_length ) {
-	index = qre.match( *this, index, &len, FALSE );
+	index = qre.match( *this, index, &len);
 	if ( index >= 0 ) {
 	    replace( index, len, str );
 	    index += slen;
