@@ -25,8 +25,9 @@
 
 #import "KWQTextArea.h"
 
-#import "KWQTextEdit.h"
 #import "KWQKHTMLPart.h"
+#import "KWQNSViewExtras.h"
+#import "KWQTextEdit.h"
 #import "WebCoreBridge.h"
 
 /*
@@ -44,15 +45,37 @@
     HARD|PHYSICAL - Text is wrapped, and text is broken into separate lines.
 */
 
+
 @interface NSView (KWQTextArea)
 - (void)_KWQ_setKeyboardFocusRingNeedsDisplay;
 @end
 
+@interface NSTextView (KWQTextArea)
+- (NSParagraphStyle *)_KWQ_typingParagraphStyle;
+- (void)_KWQ_setTypingParagraphStyle:(NSParagraphStyle *)style;
+@end
+
+@interface NSTextStorage (KWQTextArea)
+- (void)_KWQ_setBaseWritingDirection:(NSWritingDirection)direction;
+@end
+
 @interface KWQTextAreaTextView : NSTextView <KWQWidgetHolder>
 {
     QTextEdit *widget;
+    BOOL disabled;
+    BOOL editableIfEnabled;
 }
+
 - (void)setWidget:(QTextEdit *)widget;
+
+- (void)setEnabled:(BOOL)flag;
+- (BOOL)isEnabled;
+
+- (void)setEditableIfEnabled:(BOOL)flag;
+- (BOOL)isEditableIfEnabled;
+
+- (void)updateTextColor;
+
 @end
 
 @implementation KWQTextArea
@@ -64,14 +87,16 @@ const float LargeNumberForText = 1.0e7;
     NSSize size = [self frame].size;
     NSRect textFrame;
     textFrame.origin.x = textFrame.origin.y = 0;
-    if (size.width > 0 && size.height > 0)
-        textFrame.size = [NSScrollView contentSizeForFrameSize:size
-            hasHorizontalScroller:NO hasVerticalScroller:YES borderType:[self borderType]];
-    else {
+    if (size.width > 0 && size.height > 0) {
+        textFrame.size = [[self class] contentSizeForFrameSize:size
+            hasHorizontalScroller:[self hasHorizontalScroller]
+            hasVerticalScroller:[self hasVerticalScroller]
+            borderType:[self borderType]];
+    } else {
         textFrame.size.width = LargeNumberForText;
         textFrame.size.height = LargeNumberForText;
     }
-        
+
     textView = [[KWQTextAreaTextView alloc] initWithFrame:textFrame];
     [textView setRichText:NO];
     [[textView textContainer] setWidthTracksTextView:YES];
@@ -82,7 +107,7 @@ const float LargeNumberForText = 1.0e7;
     NSMutableParagraphStyle *style = [[NSMutableParagraphStyle alloc] init];
     [style setLineBreakMode:NSLineBreakByWordWrapping];
     [style setAlignment:NSLeftTextAlignment];
-    [textView setTypingAttributes:[NSDictionary dictionaryWithObject:style forKey:NSParagraphStyleAttributeName]];
+    [textView _KWQ_setTypingParagraphStyle:style];
     [style release];
     
     [textView setAutoresizingMask:NSViewWidthSizable | NSViewHeightSizable];
@@ -102,6 +127,18 @@ const float LargeNumberForText = 1.0e7;
     
     [self _createTextView];
     
+    // In WebHTMLView, we set a clip. This is not typical to do in an
+    // NSView, and while correct for any one invocation of drawRect:,
+    // it causes some bad problems if that clip is cached between calls.
+    // The cached graphics state, which clip views keep around, does
+    // cache the clip in this undesirable way. Consequently, we want to 
+    // release the GState for all clip views for all views contained in 
+    // a WebHTMLView. Here we do it for textareas used in forms.
+    // See these bugs for more information:
+    // <rdar://problem/3310943>: REGRESSION (Panther): textareas in forms sometimes draw blank (bugreporter)
+    [[self contentView] releaseGState];
+    [[self documentView] releaseGState];
+    
     return self;
 }
 
@@ -118,13 +155,16 @@ const float LargeNumberForText = 1.0e7;
 - (void)dealloc
 {
     [textView release];
+    [_font release];
     
     [super dealloc];
 }
 
 - (void)textDidChange:(NSNotification *)aNotification
 {
-    widget->textChanged();
+    if (!KWQKHTMLPart::handleKeyboardOptionTabInView(self)) {
+        widget->textChanged();
+    }
 }
 
 - (void)setWordWrap:(BOOL)f
@@ -164,7 +204,7 @@ const float LargeNumberForText = 1.0e7;
     
     [style setAlignment:NSLeftTextAlignment];
     
-    [textView setTypingAttributes:[NSDictionary dictionaryWithObject:style forKey:NSParagraphStyleAttributeName]];
+    [textView _KWQ_setTypingParagraphStyle:style];
     [style release];
     
     wrap = f;
@@ -177,9 +217,8 @@ const float LargeNumberForText = 1.0e7;
 
 - (void)setText:(NSString *)s
 {
-    //NSLog(@"extraLineFragmentTextContainer before setString: is %@", [[textView layoutManager] extraLineFragmentTextContainer]);
     [textView setString:s];
-    //NSLog(@"extraLineFragmentTextContainer after setString: is %@", [[textView layoutManager] extraLineFragmentTextContainer]);
+    [textView updateTextColor];
 }
 
 - (NSString *)text
@@ -236,12 +275,27 @@ const float LargeNumberForText = 1.0e7;
 
 - (void)setEditable:(BOOL)flag
 {
-    [textView setEditable:flag];
+    [textView setEditableIfEnabled:flag];
 }
 
 - (BOOL)isEditable
 {
-    return [textView isEditable];
+    return [textView isEditableIfEnabled];
+}
+
+- (void)setEnabled:(BOOL)flag
+{
+    if (flag == [textView isEnabled])
+        return;
+        
+    [textView setEnabled:flag];
+    
+    [self setNeedsDisplay:YES];
+}
+
+- (BOOL)isEnabled
+{
+    return [textView isEnabled];
 }
 
 - (void)setFrame:(NSRect)frameRect
@@ -249,7 +303,7 @@ const float LargeNumberForText = 1.0e7;
     [super setFrame:frameRect];
 
     if ([self wordWrap]) {
-        NSSize contentSize = [NSScrollView contentSizeForFrameSize:frameRect.size
+        NSSize contentSize = [[self class] contentSizeForFrameSize:frameRect.size
             hasHorizontalScroller:[self hasHorizontalScroller]
             hasVerticalScroller:[self hasVerticalScroller]
             borderType:[self borderType]];
@@ -351,9 +405,10 @@ static NSRange RangeOfParagraph(NSString *text, int paragraph)
 
 - (void)setFont:(NSFont *)font
 {
-    //NSLog(@"extraLineFragmentTextContainer before setFont: is %@", [[textView layoutManager] extraLineFragmentTextContainer]);
+    [font retain];
+    [_font release];
+    _font = font;
     [textView setFont:font];
-    //NSLog(@"extraLineFragmentTextContainer after setFont: is %@", [[textView layoutManager] extraLineFragmentTextContainer]);
 }
 
 - (BOOL)becomeFirstResponder
@@ -401,7 +456,20 @@ static NSRange RangeOfParagraph(NSString *text, int paragraph)
 - (void)drawRect:(NSRect)rect
 {
     [super drawRect:rect];
-    if ([KWQKHTMLPart::bridgeForWidget(widget) firstResponder] == textView) {
+    if (![textView isEnabled]) {
+        // draw a disabled bezel border
+        [[NSColor controlColor] set];
+        NSFrameRect(rect);
+        
+        rect = NSInsetRect(rect, 1, 1);
+        [[NSColor controlShadowColor] set];
+        NSFrameRect(rect);
+    
+        rect = NSInsetRect(rect, 1, 1);
+        [[NSColor textBackgroundColor] set];
+        NSRectFill(rect);
+    }
+    else if ([KWQKHTMLPart::bridgeForWidget(widget) firstResponder] == textView) {
         NSSetFocusRingStyle(NSFocusRingOnly);
         NSRectFill([self bounds]);
     }
@@ -422,6 +490,76 @@ static NSRange RangeOfParagraph(NSString *text, int paragraph)
     [textView setAlignment:alignment];
 }
 
+- (void)setBaseWritingDirection:(NSWritingDirection)direction
+{
+    // Set the base writing direction for typing.
+    NSParagraphStyle *style = [textView _KWQ_typingParagraphStyle];
+    if ([style baseWritingDirection] != direction) {
+        NSMutableParagraphStyle *mutableStyle = [style mutableCopy];
+        [mutableStyle setBaseWritingDirection:direction];
+        [textView _KWQ_setTypingParagraphStyle:mutableStyle];
+        [mutableStyle release];
+    }
+
+    // Set the base writing direction for text.
+    [[textView textStorage] _KWQ_setBaseWritingDirection:direction];
+    [textView setNeedsDisplay:YES];
+}
+
+// This is the only one of the display family of calls that we use, and the way we do
+// displaying in WebCore means this is called on this NSView explicitly, so this catches
+// all cases where we are inside the normal display machinery. (Used only by the insertion
+// point method below.)
+- (void)displayRectIgnoringOpacity:(NSRect)rect
+{
+    inDrawingMachinery = YES;
+    [super displayRectIgnoringOpacity:rect];
+    inDrawingMachinery = NO;
+}
+
+// Use the "needs display" mechanism to do all insertion point drawing in the web view.
+- (BOOL)textView:(NSTextView *)view shouldDrawInsertionPointInRect:(NSRect)rect color:(NSColor *)color turnedOn:(BOOL)drawInsteadOfErase
+{
+    // We only need to take control of the cases where we are being asked to draw by something
+    // outside the normal display machinery, and when we are being asked to draw the insertion
+    // point, not erase it.
+    if (inDrawingMachinery || !drawInsteadOfErase) {
+        return YES;
+    }
+
+    // NSTextView's insertion-point drawing code sets the rect width to 1.
+    // So we do the same thing, to affect exactly the same rectangle.
+    rect.size.width = 1;
+
+    // Call through to the setNeedsDisplayInRect implementation in NSView.
+    // If we call the one in NSTextView through the normal method dispatch
+    // we will reenter the caret blinking code and end up with a nasty crash
+    // (see Radar 3250608).
+    SEL selector = @selector(setNeedsDisplayInRect:);
+    typedef void (*IMPWithNSRect)(id, SEL, NSRect);
+    IMPWithNSRect implementation = (IMPWithNSRect)[NSView instanceMethodForSelector:selector];
+    implementation(view, selector, rect);
+
+    return NO;
+}
+
+- (NSSize)sizeWithColumns:(int)numColumns rows:(int)numRows
+{
+    // Must use font from _font field rather than from the text view's font method,
+    // because the text view will return a substituted font if the first character in
+    // the text view requires font substitution, and we don't want the size to depend on
+    // the text in the text view.
+    
+    NSSize textSize = NSMakeSize(ceil(numColumns * [_font widthOfString:@"0"]), numRows * [_font defaultLineHeightForFont]);
+    NSSize textContainerSize = NSMakeSize(textSize.width + [[textView textContainer] lineFragmentPadding] * 2, textSize.height);
+    NSSize textContainerInset = [textView textContainerInset];
+    NSSize textViewSize = NSMakeSize(textContainerSize.width + textContainerInset.width, textContainerSize.height + textContainerInset.height); 
+    return [[self class] frameSizeForContentSize:textViewSize
+        hasHorizontalScroller:[self hasHorizontalScroller]
+        hasVerticalScroller:[self hasVerticalScroller]
+        borderType:[self borderType]];
+}
+
 @end
 
 @implementation KWQTextAreaTextView
@@ -452,6 +590,8 @@ static NSString *WebContinuousSpellCheckingEnabled = @"WebContinuousSpellCheckin
     [super setContinuousSpellCheckingEnabled:
         [[self class] _isContinuousSpellCheckingEnabledForNewTextAreas]];
 
+    editableIfEnabled = YES;
+
     return self;
 }
 
@@ -485,19 +625,25 @@ static NSString *WebContinuousSpellCheckingEnabled = @"WebContinuousSpellCheckin
 
 - (BOOL)becomeFirstResponder
 {
-    BOOL become = [super becomeFirstResponder];
+    if (disabled)
+        return NO;
 
+    BOOL become = [super becomeFirstResponder];
+    
     if (become) {
         // Select all the text if we are tabbing in, but otherwise preserve/remember
         // the selection from last time we had focus (to match WinIE).
         if ([[self window] keyViewSelectionDirection] != NSDirectSelection) {
             [self selectAll:nil];
         }
+        if (!KWQKHTMLPart::currentEventIsMouseDownInWidget(widget)) {
+            [self _KWQ_scrollFrameToVisible];
+        }        
 	[self _KWQ_setKeyboardFocusRingNeedsDisplay];
 	QFocusEvent event(QEvent::FocusIn);
 	const_cast<QObject *>(widget->eventFilterObject())->eventFilter(widget, &event);
     }
-       
+
     return become;
 }
 
@@ -551,11 +697,75 @@ static NSString *WebContinuousSpellCheckingEnabled = @"WebContinuousSpellCheckin
 
 - (void)mouseDown:(NSEvent *)event
 {
+    if (disabled)
+        return;
     [super mouseDown:event];
     widget->sendConsumedMouseUp();
     widget->clicked();
 }
 
+- (void)keyDown:(NSEvent *)event
+{
+    if (disabled)
+        return;
+    WebCoreBridge *bridge = KWQKHTMLPart::bridgeForWidget(widget);
+    if ([[NSInputManager currentInputManager] hasMarkedText] || 
+	![bridge interceptKeyEvent:event toView:self]) {
+	[super keyDown:event];
+    }
+}
+
+- (void)keyUp:(NSEvent *)event
+{
+    if (disabled)
+        return;
+    WebCoreBridge *bridge = KWQKHTMLPart::bridgeForWidget(widget);
+    if (![[NSInputManager currentInputManager] hasMarkedText]) {
+	[bridge interceptKeyEvent:event toView:self];
+    }
+    // Don't call super because NSTextView will simply pass the
+    // event along the responder chain. This is arguably a bug in
+    // NSTextView; see Radar 3507083.
+}
+
+- (void)setEnabled:(BOOL)flag
+{
+    if (disabled == !flag) {
+        return;
+    }
+
+    disabled = !flag;
+    if (editableIfEnabled) {
+        [self setEditable:!disabled];
+    }
+    [self updateTextColor];
+}
+
+- (BOOL)isEnabled
+{
+    return !disabled;
+}
+
+- (void)setEditableIfEnabled:(BOOL)flag
+{
+    editableIfEnabled = flag;
+    if (!disabled) {
+        [self setEditable:editableIfEnabled];
+    }
+}
+
+- (BOOL)isEditableIfEnabled
+{
+    return editableIfEnabled;
+}
+
+- (void)updateTextColor
+{
+    // Make the text look disabled by changing its color.
+    NSColor *color = disabled ? [NSColor disabledControlTextColor] : [NSColor controlTextColor];
+    [[self textStorage] setForegroundColor:color];
+}
+
 @end
 
 @implementation NSView (KWQTextArea)
@@ -566,3 +776,64 @@ static NSString *WebContinuousSpellCheckingEnabled = @"WebContinuousSpellCheckin
 }
 
 @end
+
+@implementation NSTextView (KWQTextArea)
+
+- (NSParagraphStyle *)_KWQ_typingParagraphStyle
+{
+    NSParagraphStyle *style = [[self typingAttributes] objectForKey:NSParagraphStyleAttributeName];
+    if (style != nil) {
+        return style;
+    }
+    style = [self defaultParagraphStyle];
+    if (style != nil) {
+        return style;
+    }
+    return [NSParagraphStyle defaultParagraphStyle];
+}
+
+- (void)_KWQ_setTypingParagraphStyle:(NSParagraphStyle *)style
+{
+    NSParagraphStyle *immutableStyle = [style copy];
+    NSDictionary *attributes = [self typingAttributes];
+    if (attributes != nil) {
+        NSMutableDictionary *mutableAttributes = [attributes mutableCopy];
+        [mutableAttributes setObject:immutableStyle forKey:NSParagraphStyleAttributeName];
+        attributes = mutableAttributes;
+    } else {
+        attributes = [[NSDictionary alloc] initWithObjectsAndKeys:immutableStyle, NSParagraphStyleAttributeName, nil];
+    }
+    [immutableStyle release];
+    [self setTypingAttributes:attributes];
+    [attributes release];
+}
+
+@end
+
+@implementation NSTextStorage (KWQTextArea)
+
+- (void)_KWQ_setBaseWritingDirection:(NSWritingDirection)direction
+{
+    unsigned end = [self length];
+    NSRange range = NSMakeRange(0, end);
+    if (end != 0) {
+        [self beginEditing];
+        for (unsigned i = 0; i < end; ) {
+            NSRange effectiveRange;
+            NSParagraphStyle *style = [self attribute:NSParagraphStyleAttributeName atIndex:i longestEffectiveRange:&effectiveRange inRange:range];
+            if (style == nil) {
+                style = [NSParagraphStyle defaultParagraphStyle];
+            }
+            if ([style baseWritingDirection] != direction) {
+                NSMutableParagraphStyle *mutableStyle = [style mutableCopy];
+                [mutableStyle setBaseWritingDirection:direction];
+                [self addAttribute:NSParagraphStyleAttributeName value:mutableStyle range:effectiveRange];
+                [mutableStyle release];
+            }
+            i = NSMaxRange(effectiveRange);
+        }
+        [self endEditing];
+    }
+}
+
+@end
