@@ -31,14 +31,28 @@
 #import "KWQNSViewExtras.h"
 #import "KWQView.h"
 #import "WebCoreBridge.h"
-#import "WebCoreFirstResponderChanges.h"
+
+@interface NSString (KWQTextField)
+- (int)_KWQ_numComposedCharacterSequences;
+- (NSString *)_KWQ_truncateToNumComposedCharacterSequences:(int)num;
+@end
+
+@interface NSCell (KWQTextFieldKnowsAppKitSecrets)
+- (NSMutableDictionary *)_textAttributes;
+@end
 
 @interface KWQTextField (KWQInternal)
 - (void)setHasFocus:(BOOL)hasFocus;
 @end
 
-// KWQTextFieldCell allows us to tell when we get focus without an editor subclass.
+// KWQTextFieldCell allows us to tell when we get focus without an editor subclass,
+// and override the base writing direction.
 @interface KWQTextFieldCell : NSTextFieldCell
+{
+    NSWritingDirection baseWritingDirection;
+}
+- (void)setBaseWritingDirection:(NSWritingDirection)direction;
+- (NSWritingDirection)baseWritingDirection;
 @end
 
 // KWQTextFieldFormatter enforces a maximum length.
@@ -55,13 +69,22 @@
 // Another is hook up next and previous key views to KHTML.
 @interface KWQSecureTextField : NSSecureTextField <KWQWidgetHolder>
 {
+    QLineEdit *widget;
     BOOL inNextValidKeyView;
     BOOL inSetFrameSize;
 }
+
+- (id)initWithQLineEdit:(QLineEdit *)widget;
+
 @end
 
-// KWQSecureTextFieldCell allows us to tell when we get focus without an editor subclass.
+// KWQSecureTextFieldCell allows us to tell when we get focus without an editor subclass,
+// and override the base writing direction.
 @interface KWQSecureTextFieldCell : NSSecureTextFieldCell
+{
+    NSWritingDirection baseWritingDirection;
+}
+- (void)setBaseWritingDirection:(NSWritingDirection)direction;
 @end
 
 @implementation KWQTextField
@@ -87,7 +110,7 @@
     [field setAction:@selector(action:)];
 }
 
-- initWithFrame:(NSRect)frame
+-(id)initWithFrame:(NSRect)frame
 {
     [super initWithFrame:frame];
     formatter = [[KWQTextFieldFormatter alloc] init];
@@ -96,14 +119,23 @@
     return self;
 }
 
-- initWithQLineEdit:(QLineEdit *)w 
+-(id)initWithQLineEdit:(QLineEdit *)w 
 {
     widget = w;
     return [self init];
 }
 
+-(void)invalidate
+{
+    widget = NULL;
+}
+
 - (void)action:sender
 {
+    if (!widget) {
+	return;
+    }
+
     widget->returnPressed();
 }
 
@@ -143,11 +175,12 @@
         [secureField removeFromSuperview];
     } else {
         if (secureField == nil) {
-            secureField = [[KWQSecureTextField alloc] init];
+            secureField = [[KWQSecureTextField alloc] initWithQLineEdit:widget];
             [secureField setFormatter:formatter];
             [secureField setFont:[self font]];
             [secureField setEditable:[self isEditable]];
             [secureField setSelectable:[self isSelectable]];
+            [[secureField cell] setBaseWritingDirection:[[self cell] baseWritingDirection]];
             [self setUpTextField:secureField];
             [self updateSecureFieldFrame];
         }
@@ -200,8 +233,8 @@
 - (void)setMaximumLength:(int)len
 {
     NSString *oldValue = [self stringValue];
-    if ((int)[oldValue length] > len) {
-        [self setStringValue:[oldValue substringToIndex:len]];
+    if ([oldValue _KWQ_numComposedCharacterSequences] > len) {
+        [self setStringValue:[oldValue _KWQ_truncateToNumComposedCharacterSequences:len]];
     }
     [formatter setMaximumLength:len];
 }
@@ -221,66 +254,119 @@
     edited = ed;
 }
 
-- (void)controlTextDidBeginEditing:(NSNotification *)notification
+-(void)controlTextDidBeginEditing:(NSNotification *)notification
 {
+    if (!widget) {
+	return;
+    }
+
     WebCoreBridge *bridge = KWQKHTMLPart::bridgeForWidget(widget);
     [bridge controlTextDidBeginEditing:notification];
 }
 
-- (void)controlTextDidEndEditing:(NSNotification *)notification
+-(void)controlTextDidEndEditing:(NSNotification *)notification
 {
-    [self setHasFocus:NO];
+    if (!widget) {
+	return;
+    }
 
     WebCoreBridge *bridge = KWQKHTMLPart::bridgeForWidget(widget);
     [bridge controlTextDidEndEditing:notification];
+    
+    [self setHasFocus:NO];
 }
 
-- (void)controlTextDidChange:(NSNotification *)notification
+-(void)controlTextDidChange:(NSNotification *)notification
 {
+    if (!widget) {
+	return;
+    }
+    
+    if (KWQKHTMLPart::handleKeyboardOptionTabInView(self)) {
+        return;
+    }
+
     WebCoreBridge *bridge = KWQKHTMLPart::bridgeForWidget(widget);
     [bridge controlTextDidChange:notification];
-
+    
     edited = YES;
     widget->textChanged();
 }
 
-- (BOOL)control:(NSControl *)control textShouldBeginEditing:(NSText *)fieldEditor
+-(BOOL)control:(NSControl *)control textShouldBeginEditing:(NSText *)fieldEditor
 {
+    if (!widget) {
+	return NO;
+    }
+
     WebCoreBridge *bridge = KWQKHTMLPart::bridgeForWidget(widget);
+
+    // In WebHTMLView, we set a clip. This is not typical to do in an
+    // NSView, and while correct for any one invocation of drawRect:,
+    // it causes some bad problems if that clip is cached between calls.
+    // The cached graphics state, which some views keep around, does
+    // cache the clip in this undesirable way. Consequently, we want to 
+    // turn off this caching for all views contained in a WebHTMLView that
+    // would otherwise do it. Here we turn it off for the editor (NSTextView)
+    // used for text fields in forms and the clip view it's embedded in.
+    // See bug 3457875 and 3310943 for more context.
+    [fieldEditor releaseGState];
+    [[fieldEditor superview] releaseGState];
+
     return [bridge control:control textShouldBeginEditing:fieldEditor];
 }
 
-- (BOOL)control:(NSControl *)control textShouldEndEditing:(NSText *)fieldEditor
+-(BOOL)control:(NSControl *)control textShouldEndEditing:(NSText *)fieldEditor
 {
+    if (!widget) {
+	return NO;
+    }
+
     WebCoreBridge *bridge = KWQKHTMLPart::bridgeForWidget(widget);
     return [bridge control:control textShouldEndEditing:fieldEditor];
 }
 
-- (BOOL)control:(NSControl *)control didFailToFormatString:(NSString *)string errorDescription:(NSString *)error
+-(BOOL)control:(NSControl *)control didFailToFormatString:(NSString *)string errorDescription:(NSString *)error
 {
+    if (!widget) {
+	return NO;
+    }
+
     WebCoreBridge *bridge = KWQKHTMLPart::bridgeForWidget(widget);
     return [bridge control:control didFailToFormatString:string errorDescription:error];
 }
 
-- (void)control:(NSControl *)control didFailToValidatePartialString:(NSString *)string errorDescription:(NSString *)error
+-(void)control:(NSControl *)control didFailToValidatePartialString:(NSString *)string errorDescription:(NSString *)error
 {
+    if (!widget) {
+	return;
+    }
+
     WebCoreBridge *bridge = KWQKHTMLPart::bridgeForWidget(widget);
     [bridge control:control didFailToValidatePartialString:string errorDescription:error];
 }
 
-- (BOOL)control:(NSControl *)control isValidObject:(id)obj
+-(BOOL)control:(NSControl *)control isValidObject:(id)obj
 {
+    if (!widget) {
+	return NO;
+    }
+
     WebCoreBridge *bridge = KWQKHTMLPart::bridgeForWidget(widget);
     return [bridge control:control isValidObject:obj];
 }
 
-- (BOOL)control:(NSControl *)control textView:(NSTextView *)textView doCommandBySelector:(SEL)commandSelector
+-(BOOL)control:(NSControl *)control textView:(NSTextView *)textView doCommandBySelector:(SEL)commandSelector
 {
+    if (!widget) {
+	return NO;
+    }
+
     WebCoreBridge *bridge = KWQKHTMLPart::bridgeForWidget(widget);
     return [bridge control:control textView:textView doCommandBySelector:commandSelector];
 }
 
-- (NSString *)stringValue
+-(NSString *)stringValue
 {
     if ([secureField superview]) {
         return [secureField stringValue];
@@ -288,38 +374,48 @@
     return [super stringValue];
 }
 
-- (void)setStringValue:(NSString *)string
+-(void)setStringValue:(NSString *)string
 {
-    int maxLength = [formatter maximumLength];
-    if ((int)[string length] > maxLength) {
-        string = [string substringToIndex:maxLength];
+    if (!widget) {
+	return;
     }
+
+    int maxLength = [formatter maximumLength];
+    string = [string _KWQ_truncateToNumComposedCharacterSequences:maxLength];
     [secureField setStringValue:string];
     [super setStringValue:string];
     widget->textChanged();
 }
 
-- (void)setFont:(NSFont *)font
+-(void)setFont:(NSFont *)font
 {
     [secureField setFont:font];
     [super setFont:font];
 }
 
-- (NSView *)nextKeyView
+-(NSView *)nextKeyView
 {
+    if (!widget) {
+	return [super nextKeyView];
+    }
+
     return inNextValidKeyView
         ? KWQKHTMLPart::nextKeyViewForWidget(widget, KWQSelectingNext)
         : [super nextKeyView];
 }
 
-- (NSView *)previousKeyView
+-(NSView *)previousKeyView
 {
-   return inNextValidKeyView
+    if (!widget) {
+	return [super previousKeyView];
+    }
+
+    return inNextValidKeyView
         ? KWQKHTMLPart::nextKeyViewForWidget(widget, KWQSelectingPrevious)
         : [super previousKeyView];
 }
 
-- (NSView *)nextValidKeyView
+-(NSView *)nextValidKeyView
 {
     inNextValidKeyView = YES;
     NSView *view = [super nextValidKeyView];
@@ -327,7 +423,7 @@
     return view;
 }
 
-- (NSView *)previousValidKeyView
+-(NSView *)previousValidKeyView
 {
     inNextValidKeyView = YES;
     NSView *view = [super previousValidKeyView];
@@ -335,6 +431,11 @@
     return view;
 }
 
+- (BOOL)acceptsFirstResponder
+{
+    return [self isEnabled];
+}
+
 - (BOOL)becomeFirstResponder
 {
     if ([self passwordMode]) {
@@ -355,18 +456,91 @@
     return widget;
 }
 
-- (void)fieldEditorDidMouseDown:(NSEvent *)event
-{
-    widget->sendConsumedMouseUp();
-    widget->clicked();
-}
-
 - (void)setAlignment:(NSTextAlignment)alignment
 {
     [secureField setAlignment:alignment];
     [super setAlignment:alignment];
 }
 
+// This is the only one of the display family of calls that we use, and the way we do
+// displaying in WebCore means this is called on this NSView explicitly, so this catches
+// all cases where we are inside the normal display machinery. (Used only by the insertion
+// point method below.)
+- (void)displayRectIgnoringOpacity:(NSRect)rect
+{
+    inDrawingMachinery = YES;
+    [super displayRectIgnoringOpacity:rect];
+    inDrawingMachinery = NO;
+}
+
+// Use the "needs display" mechanism to do all insertion point drawing in the web view.
+- (BOOL)textView:(NSTextView *)view shouldDrawInsertionPointInRect:(NSRect)rect color:(NSColor *)color turnedOn:(BOOL)drawInsteadOfErase
+{
+    // We only need to take control of the cases where we are being asked to draw by something
+    // outside the normal display machinery, and when we are being asked to draw the insertion
+    // point, not erase it.
+    if (inDrawingMachinery || !drawInsteadOfErase) {
+        return YES;
+    }
+
+    // NSTextView's insertion-point drawing code sets the rect width to 1.
+    // So we do the same thing, to affect exactly the same rectangle.
+    rect.size.width = 1;
+
+    // Call through to the setNeedsDisplayInRect implementation in NSView.
+    // If we call the one in NSTextView through the normal method dispatch
+    // we will reenter the caret blinking code and end up with a nasty crash
+    // (see Radar 3250608).
+    SEL selector = @selector(setNeedsDisplayInRect:);
+    typedef void (*IMPWithNSRect)(id, SEL, NSRect);
+    IMPWithNSRect implementation = (IMPWithNSRect)[NSView instanceMethodForSelector:selector];
+    implementation(view, selector, rect);
+
+    return NO;
+}
+
+- (BOOL)textView:(NSTextView *)view shouldHandleEvent:(NSEvent *)event
+{
+    if (!widget) {
+	return YES;
+    }
+
+    NSEventType type = [event type];
+    if ((type == NSKeyDown || type == NSKeyUp) && 
+	![[NSInputManager currentInputManager] hasMarkedText]) {
+        WebCoreBridge *bridge = KWQKHTMLPart::bridgeForWidget(widget);
+        BOOL intercepted = [bridge interceptKeyEvent:event toView:view];
+        // Always return NO for key up events because we don't want them
+        // passed along the responder chain. This is arguably a bug in
+        // NSTextView; see Radar 3507083.
+        return type != NSKeyUp && !intercepted;
+    }
+    return YES;
+}
+
+- (void)textView:(NSTextView *)view didHandleEvent:(NSEvent *)event
+{
+    if (!widget) {
+	return;
+    }
+    if ([event type] == NSLeftMouseUp) {
+        widget->sendConsumedMouseUp();
+        widget->clicked();
+    }
+}
+
+- (void)setBaseWritingDirection:(NSWritingDirection)direction
+{
+    KWQTextFieldCell *cell = [self cell];
+    if ([cell baseWritingDirection] != direction) {
+        [cell setBaseWritingDirection:direction];
+        [[secureField cell] setBaseWritingDirection:direction];
+
+        // One call to setNeedsDisplay will take care of both text fields.
+        [self setNeedsDisplay:YES];
+    }
+}
+
 @end
 
 @implementation KWQTextField (KWQInternal)
@@ -401,25 +575,32 @@
     }
 }
 
-- (void)setHasFocus:(BOOL)hasFocus
+- (void)setHasFocus:(BOOL)nowHasFocus
 {
-    if (hasFocus == _hasFocus) {
+    if (!widget) {
+	return;
+    }
+
+    if (nowHasFocus == hasFocus) {
         return;
     }
 
-    _hasFocus = hasFocus;
+    hasFocus = nowHasFocus;
 
-    if (hasFocus) {
+    if (nowHasFocus) {
         // Select all the text if we are tabbing in, but otherwise preserve/remember
         // the selection from last time we had focus (to match WinIE).
         if ([[self window] keyViewSelectionDirection] != NSDirectSelection) {
             lastSelectedRange.location = NSNotFound;
         }
+
         if (lastSelectedRange.location != NSNotFound) {
             [self setSelectedRange:lastSelectedRange];
         }
-
-        [self _KWQ_scrollFrameToVisible];
+        
+        if (!KWQKHTMLPart::currentEventIsMouseDownInWidget(widget)) {
+            [self _KWQ_scrollFrameToVisible];
+        }
 
         QFocusEvent event(QEvent::FocusIn);
         const_cast<QObject *>(widget->eventFilterObject())->eventFilter(widget, &event);
@@ -462,6 +643,30 @@
     [(KWQTextField *)delegate setHasFocus:YES];
 }
 
+- (void)setBaseWritingDirection:(NSWritingDirection)direction
+{
+    baseWritingDirection = direction;
+}
+
+- (NSWritingDirection)baseWritingDirection
+{
+    return baseWritingDirection;
+}
+
+- (NSMutableDictionary *)_textAttributes
+{
+    NSMutableDictionary *attributes = [super _textAttributes];
+    NSParagraphStyle *style = [attributes objectForKey:NSParagraphStyleAttributeName];
+    ASSERT(style != nil);
+    if ([style baseWritingDirection] != baseWritingDirection) {
+        NSMutableParagraphStyle *mutableStyle = [style mutableCopy];
+        [mutableStyle setBaseWritingDirection:baseWritingDirection];
+        [attributes setObject:mutableStyle forKey:NSParagraphStyleAttributeName];
+        [mutableStyle release];
+    }
+    return attributes;
+}
+
 @end
 
 @implementation KWQTextFieldFormatter
@@ -496,7 +701,7 @@
 
 - (BOOL)isPartialStringValid:(NSString *)partialString newEditingString:(NSString **)newString errorDescription:(NSString **)error
 {
-    if ((int)[partialString length] > maxLength) {
+    if ([partialString _KWQ_numComposedCharacterSequences] > maxLength) {
         *newString = nil;
         return NO;
     }
@@ -513,6 +718,12 @@
 
 @implementation KWQSecureTextField
 
+-(id)initWithQLineEdit:(QLineEdit *)w 
+{
+    widget = w;
+    return [self init];
+}
+
 // Can't use setCellClass: because NSSecureTextField won't let us (for no good reason).
 + (Class)cellClass
 {
@@ -606,12 +817,6 @@
     return [(KWQTextField *)[self delegate] widget];
 }
 
-- (void)fieldEditorDidMouseDown:(NSEvent *)event
-{
-    ASSERT([[self delegate] isKindOfClass:[KWQTextField class]]);
-    [[self delegate] fieldEditorDidMouseDown:event];
-}
-
 - (void)textDidEndEditing:(NSNotification *)notification
 {
     [super textDidEndEditing:notification];
@@ -633,6 +838,31 @@
     }
 }
 
+- (BOOL)textView:(NSTextView *)view shouldHandleEvent:(NSEvent *)event
+{
+    if (!widget) {
+	return YES;
+    }
+
+    if (([event type] == NSKeyDown || [event type] == NSKeyUp) &&
+	![[NSInputManager currentInputManager] hasMarkedText]) {
+        WebCoreBridge *bridge = KWQKHTMLPart::bridgeForWidget(widget);
+        return ![bridge interceptKeyEvent:event toView:view];
+    }
+    return YES;
+}
+
+- (void)textView:(NSTextView *)view didHandleEvent:(NSEvent *)event
+{
+    if (!widget) {
+	return;
+    }
+    if ([event type] == NSLeftMouseUp) {
+        widget->sendConsumedMouseUp();
+        widget->clicked();
+    }
+}
+
 @end
 
 @implementation KWQSecureTextFieldCell
@@ -651,4 +881,55 @@
     [(KWQTextField *)[delegate delegate] setHasFocus:YES];
 }
 
+- (void)setBaseWritingDirection:(NSWritingDirection)direction
+{
+    baseWritingDirection = direction;
+}
+
+- (NSMutableDictionary *)_textAttributes
+{
+    NSMutableDictionary *attributes = [super _textAttributes];
+    NSParagraphStyle *style = [attributes objectForKey:NSParagraphStyleAttributeName];
+    ASSERT(style != nil);
+    if ([style baseWritingDirection] != baseWritingDirection) {
+        NSMutableParagraphStyle *mutableStyle = [style mutableCopy];
+        [mutableStyle setBaseWritingDirection:baseWritingDirection];
+        [attributes setObject:mutableStyle forKey:NSParagraphStyleAttributeName];
+        [mutableStyle release];
+    }
+    return attributes;
+}
+
+@end
+
+@implementation NSString (KWQTextField)
+
+- (int)_KWQ_numComposedCharacterSequences
+{
+    unsigned i = 0;
+    unsigned l = [self length];
+    int num = 0;
+    while (i < l) {
+        i = NSMaxRange([self rangeOfComposedCharacterSequenceAtIndex:i]);
+        ++num;
+    }
+    return num;
+}
+
+- (NSString *)_KWQ_truncateToNumComposedCharacterSequences:(int)num
+{
+    unsigned i = 0;
+    unsigned l = [self length];
+    if (l == 0) {
+        return self;
+    }
+    for (int j = 0; j < num; j++) {
+        i = NSMaxRange([self rangeOfComposedCharacterSequenceAtIndex:i]);
+        if (i >= l) {
+            return self;
+        }
+    }
+    return [self substringToIndex:i];
+}
+
 @end
