@@ -1033,6 +1033,15 @@ static HTMLFormElementImpl *formElementFromDOMElement(id <WebDOMElement>element)
     return _part->referrer().getNSString();
 }
 
+- (NSString *)domain
+{
+    DocumentImpl *doc = _part->xmlDocImpl();
+    if (doc && doc->isHTMLDocument()) {
+        return doc->domain().string().getNSString();
+    }
+    return nil;
+}
+
 + (NSString *)stringWithData:(NSData *)data textEncoding:(CFStringEncoding)textEncoding
 {
     if (textEncoding == kCFStringEncodingInvalidId || textEncoding == kCFStringEncodingISOLatin1) {
